using Microsoft.FluentUI.AspNetCore.Components;
using FluentBlazorAuth.Components;
using Microsoft.AspNetCore.Authentication.Cookies;
using FluentBlazorAuth.Data;
using Microsoft.EntityFrameworkCore;
using FluentBlazorAuth.Services;
using FluentBlazorAuth.States;
using Microsoft.AspNetCore.Components.Authorization;

namespace FluentBlazorAuth;

public class Program
{
    public static void Main(string[] args)
    {
        var builder = WebApplication.CreateBuilder(args);

        // Add services to the container.
        builder.Services.AddRazorComponents()
            .AddInteractiveServerComponents();
        builder.Services.AddFluentUIComponents();

        #region Authentication
        builder.Services.AddAuthentication(CookieAuthenticationDefaults.AuthenticationScheme)
        .AddCookie(option => 
        {
            option.Cookie.Name = "auth_token";
            option.LoginPath = "/login";
            option.Cookie.MaxAge = TimeSpan.FromMinutes(30);
            option.AccessDeniedPath = "/access-denied";
        });
        builder.Services.AddAuthorization();
        builder.Services.AddCascadingAuthenticationState();
        #endregion

        #region AuthTest
        builder.Services.AddScoped<IUserAccountService, UserAccountService>();
        builder.Services.AddScoped<AuthenticationStateProvider, CustomAuthenticationStateProvider>();
        builder.Services.AddHttpContextAccessor();
        #endregion

        #region Database
        var connectionString = builder.Configuration.GetConnectionString("DefaultConnection");
        builder.Services.AddDbContext<AppDbContext>(options =>
        {
            options.UseMySql(connectionString, ServerVersion.AutoDetect(connectionString)); 
        });
        #endregion

        var app = builder.Build();

        // Configure the HTTP request pipeline.
        if (!app.Environment.IsDevelopment())
        {
            app.UseExceptionHandler("/Error");
            // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
            app.UseHsts();
        }

        app.UseHttpsRedirection();

        app.UseStaticFiles();
        app.UseAntiforgery();

        #region Authentication
        app.UseAuthentication();
        app.UseAuthorization();
        #endregion

        app.MapRazorComponents<App>()
            .AddInteractiveServerRenderMode();

        app.Run();
    }
}
